#include "SeriesBase.h"
#include "ReadWorker.h"
#include "Image.h"
#include "myGobal.h"

#include "dcmtk/dcmimgle/dcmimage.h"

#include <qmainwindow.h>
#include <QEventLoop>
#include <QThread>
#include <QObject>
#include <QSlider>

SeriesBase::SeriesBase()
{
	SeriesBase::volume = 0;
	volumeSlices = 0;
	slider = new QSlider;
	slider->setMinimum(0);
	slider->setOrientation(Qt::Vertical);
	slider->setAttribute(Qt::WA_StyledBackground, true);
	slider->setStyleSheet("background-color: rgba(200,200, 200,0)");

}

SeriesBase::~SeriesBase()
{
	qDeleteAll(dicoms.values());
	dicoms.clear();
	deleteVolume();
}

void SeriesBase::readSeriesUid(QString &uid)
{
	if (UID == uid)
		return;
	UID = uid;
	qDeleteAll(dicoms.values());
	dicoms.clear();
	deleteVolume();

	SeriesBase::curXYFrame = 0;
	SeriesBase::curXZFrame = 0;
	SeriesBase::curYZFrame = 0;

	ReadWorker *worker = new ReadWorker("");
	worker->curUid = uid;
	QThread *thread = new QThread();
	QEventLoop loop;// ѭ

	QObject::connect(thread, SIGNAL(started()), worker, SLOT(readSeries())); // ߳̿ʼִworker->checkDicomsFileDir()
	QObject::connect(worker, SIGNAL(progress(int)), Rad_G::m_mainWin, SLOT(setProgressBarValue(int)));

	QObject::connect(worker, SIGNAL(finish()), worker, SLOT(deleteLater()));// ִɣworker
	QObject::connect(worker, SIGNAL(destroyed(QObject*)), thread, SLOT(quit()));// worker ɣ Ƴ߳
	QObject::connect(thread, SIGNAL(finished()), &loop, SLOT(quit())); // ˳̣߳ ˳ѭ
	QObject::connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater())); // ˳̣߳ ߳

	worker->moveToThread(thread); // worker ƶ߳
	thread->start(); // ʼ߳
	loop.exec(); // ȴ߳˳

	if (!isNormal())
		return;
	dicomsSize = dicoms.size();

	createVolume();
}

void SeriesBase::append(Image *dicom)
{
	if (dicom && dicom->isNormal()) {
		if (UID != dicom->getSeriesUID())
		{
			delete dicom;dicom = 0;
			return;
		}

		if (dicoms.isEmpty()) {
			dicom->getWindow(winCenter, winWidth);
			dicom->getWindow(defCenter, defWidth);
			dicom->getImageSize(imageWidth, imageHeight);
			//mzadd 
			dicom->getPatienVsl(patientID, patientName, patientBirth, patientSex, patientAge); 
			dicom->getDcmName();
			dicom->getsliceThickness(sliceThickness);
		}
		dicoms.insert(dicom->getInstanceNumber(), dicom);
	}
}

bool SeriesBase::isNormal()
{
	return !dicoms.isEmpty();
}

void SeriesBase::setColor(int color)
{
	Image::setColor(color);
}

void SeriesBase::setRoiWindow(const QRectF & rect)
{
	if (dicoms.isEmpty()) return;

	Image *image = dicoms.values().at(curXYFrame);
	image->setRoiWindow(rect);
	image->getWindow(winCenter, winWidth);
}

void SeriesBase::createVolume()
{
	dicoms.values().first()->isMonochrome() ? hasTreeView = true : hasTreeView = false;
	if (!hasTreeView)
		return;

	 volumeSlices = dicomsSize;
	size_t length = imageWidth*imageHeight;

	volume = new const qint32*[dicomsSize];

	quint8 *utemp8 = 0;
	qint8 *temp8 = 0;
	quint16 *utemp16 = 0;
	qint16 *temp16 = 0;
	quint32 *utemp32 = 0;
	qint32 *temp32 = 0;

	qint32 *temp = 0;

	for (int i = 0; i < dicomsSize; ++i)
	{
		const DiPixel *pixel = dicoms.values().at(i)->getInternalPtr();
		EP_Representation rest = pixel->getRepresentation();
		if (pixel->getCount() != length)
		{
			hasTreeView = false; volumeSlices = i;
			return;
		}

		temp = new qint32[length];
		volume[i] = temp;

		switch (rest)
		{
		case EPR_Uint8:
			utemp8 = (quint8*)pixel->getData();
			for (int j = 0; j < length; ++j)
				temp[i] = utemp8[i];
			break;
		case EPR_Sint8:
			temp8 = (qint8*)pixel->getData();
			for (int j = 0; j < length; ++j)
			{
				temp[j] = temp8[j];
			}
			break;
		case EPR_Uint16:
			utemp16 = (quint16*)pixel->getData();
			for (int j = 0; j < length; ++j)
				temp[j] = utemp16[j];	
			break;
		case EPR_Sint16:
			temp16 = (qint16*)pixel->getData();
			for (int j = 0; j < length; ++j)
			{
				temp[j] = temp16[j];
			}
			break;
		case EPR_Uint32:
			utemp32 = (quint32*)pixel->getData();
			for (int j = 0; j < length; ++j)
			{
				temp[j] = utemp32[j];
			}
			break;
		case EPR_Sint32:
			temp32 = (qint32*)pixel->getData();
			for (int j = 0; j < length; ++j)
			{
				temp[j] = temp32[j];
			}
			break;
		}
	}
}

void SeriesBase::deleteVolume()
{
	if (volume == 0)
		return;

	for (int i = 0; i < volumeSlices; ++i)
	{
		if (volume[i] != 0)
		{
			delete[] volume[i];
			volume[i] = 0;
		}
	}
	
	delete[] volume;
	volume = 0;
	volumeSlices = 0;
}

void SeriesBase::update()
{
	if(slider)
		slider->setMaximum(getFrameCount() - 1);
}


QString SeriesBase::UID; //Ψһʶ
QMap<int, Image*> SeriesBase::dicoms;// 
const qint32** SeriesBase::volume=0; // άݣYZXZͼ
int SeriesBase::volumeSlices=0; // άqint32*[]ĸYZXZͼ

int SeriesBase::curXYFrame=0;// ǰͼ
int SeriesBase::curXZFrame=0;
int SeriesBase::curYZFrame=0;

bool SeriesBase::hasTreeView=false; // ɫûͼ

double SeriesBase::winCenter; // λ
double SeriesBase::winWidth; // 
double SeriesBase::defCenter;// Ĭϴλ
double SeriesBase::defWidth;
int SeriesBase::imageWidth = 0;
int SeriesBase::imageHeight = 0;
int SeriesBase::dicomsSize = 0;

//mz add-----------------------------------------------
//
QString	SeriesBase::patientID = "";
//
QString	SeriesBase::patientName = "";
//
QDate SeriesBase::patientBirth;
//Ա
QString	SeriesBase::patientSex = "";
//
QString	SeriesBase::patientAge = "";
//
QString	SeriesBase::DcmName = "";

double SeriesBase::sliceThickness;