#include "XZSeries.h"
#include "Image.h"
#include <QSlider>


XZSeries::XZSeries():SeriesBase()
{
	vt = SeriesBase::XZ;
}

XZSeries::~XZSeries()
{
}

void XZSeries::nextFrame()
{
	++curXZFrame;
	curXZFrame >= imageHeight ? curXZFrame = 0 : curXZFrame = curXZFrame;
	slider->setValue(curXZFrame);
}

void XZSeries::prevFrame()
{
	--curXZFrame;
	curXZFrame < 0 ? curXZFrame = imageHeight - 1 : curXZFrame = curXZFrame;
	slider->setValue(curXZFrame);
}

void XZSeries::gotoFrame(int index)
{
	index < 0 ? index = 0 : index = index;
	index >= imageHeight ? index = imageHeight - 1 : index = index;
	curXZFrame = index;
	slider->setValue(curXZFrame);
}

void XZSeries::gotoFrame_2(int index)
{
	index < 0 ? index = 0 : index = index;
	index >= imageHeight ? index = imageHeight - 1 : index = index;
	curXZFrame = index;
}

bool XZSeries::getPixmap(QPixmap & pixmap)
{
	if(!hasTreeView)
		return false;

	size_t length = imageWidth * dicomsSize;
	uchar *pDIB = new uchar[length];
	memset(pDIB, 0, sizeof(uchar)*length);
	double center = winCenter;
	double width = winWidth;
	double factor = 255 / width;
	double lower = center - width / 2;

	/*>>>>>>>>>>>>>ֵ*/
	for (uint y = 0; y < dicomsSize; ++y) {
		const qint32 *ptr = volume[y];
		int idx = curXZFrame * imageWidth;
		int bmpIdx = (dicomsSize - 1 - y) * imageWidth;
		if (ptr) {
			for (uint x = 0; x < imageWidth; ++x) {
				qint32 val = ptr[idx + x];
				if (val > lower + width) pDIB[bmpIdx + x] = 255;
				else if (val > lower) pDIB[bmpIdx + x] = (val - lower)*factor;
			}
		}
	}

	bool res = Image::UcharArrayToPixmap(pDIB, imageWidth, dicomsSize, length, pixmap, 8);
	delete pDIB;
	return res;
}

int XZSeries::getCurFrame()
{
	return curXZFrame;
}

int XZSeries::getFrameCount()
{
	if (!hasTreeView)
		return 0;
	return imageHeight;
}

bool XZSeries::getRectSize(int & x_width, int & y_height)
{
	if (isNormal() && hasTreeView)
	{
		x_width = imageWidth;
		y_height = dicoms.size(); 
		return true;
	}
	return false;
}

QString XZSeries::getCurFrameName()
{
	if (isNormal())
	{
		DcmName = dicoms.values().at(curXYFrame)->getDcmName();
		return DcmName;
	}
	return "";
}

void XZSeries::getPatienVsl(QString & mID, QString & mName, QDate & mBirth, QString & mSex, QString & mAge)
{	
	//
	mID = patientID;
	//
	mName = patientName;
	//
	mBirth = patientBirth;
	//Ա
	mSex = patientSex;
	//
	mAge = patientAge;
}

bool XZSeries::getPixSpacing(double & xs, double & ys)
{
	if (isNormal() && hasTreeView)
	{
		double d;
		return dicoms.values().first()->getPixSpacing(xs, d, ys);
	}
	return false;
}

QString XZSeries::getPixelValue(int x, int y)
{
	if (isNormal()&&hasTreeView)
	{
		if (y < 0) y = 0;
		if (y >= dicomsSize) y = dicomsSize - 1;
		return dicoms.values().at(y)->getPixInfo(x, curXZFrame);
	}
	return "";
}
